@@ -19,7 +19,7 @@ class From source target where
   -- > -- Avoid this:
   -- > from (x :: s)
   -- >
-  -- > -- Prefer this:
+  -- > -- Prefer this (using [@TypeApplications@](https://downloads.haskell.org/ghc/9.6.1/docs/users_guide/exts/type_applications.html) language extension):
   -- > from @s x
   --
   -- The default implementation of this method simply calls 'Coerce.coerce',
@@ -31,5 +31,5 @@ class From source target where
   -- >>> instance From Name String
   -- >>> instance From String Name
   from :: source -> target
-  default from :: Coerce.Coercible source target => source -> target
+  default from :: (Coerce.Coercible source target) => source -> target
   from = Coerce.coerce
